/*
 * Copyright (c) 2013, Expedia Affiliate Network
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that redistributions of source code
 * retain the above copyright notice, these conditions, and the following
 * disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those
 * of the authors and should not be interpreted as representing official policies,
 * either expressed or implied, of the Expedia Affiliate Network or Expedia Inc.
 */

package com.ean.config;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This class can be used via its {@link com.ean.config.EanConfiguration#getEanProperties()} method to get a
 * PropertiesConfiguration that can be used to load properties from a file called ean.properties (by default).
 * By default the PropertiesConfiguration uses a FileChanged reloading strategy in order to reload the properties
 * periodically.
 */
public final class EanConfiguration {

    /**
     * The system property to set if a custom ean.properties file is to be used. Should be an accessible path to a
     * properties file holding the necessary configuration for the system.
     * Set the property by adding -Dean.properties=[file path].
     * Relies on {@link org.apache.commons.configuration.PropertiesConfiguration} to load the configuration, and so
     * can load remote urls if so configured.
     */
    public static final String EAN_PROPERTIES_SYSTEM_PROPERTY = "ean.properties";

    /**
     * The default file name for the ean.properties file. Will look on the PATH for "ean.properties".
     */
    public static final String EAN_PROPERTIES_FILE_NAME = "ean.properties";

    private static final Log LOG = LogFactory.getLog(EanConfiguration.class);

    private static final String SYSTEM_PROPERTY_NOT_SET_MESSAGE
        = "Java system property '%s' not set, defaulting to searching for a file named '%s' on the classpath.";

    private static final String SYSTEM_PROPERTY_SET_MESSAGE
        = "Java system property '%s' set to value: %s";

    private static final String NO_EAN_PROPERTIES_FOUND_MESSAGE
        = "No loadable configuration file found at %s, resorting to defaults.";

    private static final PropertiesConfiguration EAN_PROPERTIES;

    static {

        final String eanPropertiesSystem = System.getProperty(EAN_PROPERTIES_SYSTEM_PROPERTY);

        EAN_PROPERTIES = getPropertiesConfiguration(eanPropertiesSystem);

    }

    /**
     * Private, no-op constructor to prevent instantiation.
     */
    private EanConfiguration() {
        // see javadoc.
    }

    private static PropertiesConfiguration getPropertiesConfiguration(final String eanPropertiesSystem) {
        final String eanPropertiesFileName;
        if (eanPropertiesSystem == null) {
            if (LOG.isWarnEnabled()) {
                LOG.warn(String.format(
                    SYSTEM_PROPERTY_NOT_SET_MESSAGE,
                    EAN_PROPERTIES_SYSTEM_PROPERTY,
                    EAN_PROPERTIES_FILE_NAME));
            }
            eanPropertiesFileName = EAN_PROPERTIES_FILE_NAME;
        } else {
            if (LOG.isInfoEnabled()) {
                LOG.info(String.format(
                    SYSTEM_PROPERTY_SET_MESSAGE,
                    EAN_PROPERTIES_SYSTEM_PROPERTY,
                    eanPropertiesSystem));
            }
            eanPropertiesFileName = eanPropertiesSystem;
        }

        PropertiesConfiguration configuration;

        try {
            configuration = new PropertiesConfiguration(eanPropertiesFileName);
            configuration.setReloadingStrategy(new FileChangedReloadingStrategy());
        } catch (ConfigurationException ce) {
            if (LOG.isWarnEnabled()) {
                LOG.warn(String.format(NO_EAN_PROPERTIES_FOUND_MESSAGE, eanPropertiesFileName));
            }
            configuration = new PropertiesConfiguration();
        }
        return configuration;

    }

    /**
     * The main way to retrieve configuration.
     * @return Returns the properties configuration object which contains the various properties configured for
     * ean.properties.
     */
    public static Configuration getEanProperties() {
        return EAN_PROPERTIES;
    }

}
